@@ -1,7 +1,7 @@
 /*
  * libeio implementation
  *
- * Copyright (c) 2007,2008,2009,2010,2011 Marc Alexander Lehmann <libeio@schmorp.de>
+ * Copyright (c) 2007,2008,2009,2010,2011,2012 Marc Alexander Lehmann <libeio@schmorp.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modifica-
@@ -210,7 +210,7 @@ static void eio_destroy (eio_req *req);
   #define D_NAME(entp) entp->d_name
 
   /* POSIX_SOURCE is useless on bsd's, and XOPEN_SOURCE is unreliable there, too */
-  #if __FreeBSD__ || defined __NetBSD__ || defined __OpenBSD__
+  #if __FreeBSD__ || __NetBSD__ || __OpenBSD__
     #define _DIRENT_HAVE_D_TYPE /* sigh */
     #define D_INO(de) (de)->d_fileno
     #define D_NAMLEN(de) (de)->d_namlen
@@ -1104,8 +1104,9 @@ eio__readahead (int fd, off_t offset, size_t count, etp_worker *self)
 
   FUBd;
 
-  errno = 0;
-  return count;
+  /* linux's readahead basically only fails for EBADF or EINVAL (not mmappable) */
+  /* but not for e.g. EIO or eof, so we also never fail */
+  return 0;
 }
 
 #endif
@@ -1150,7 +1151,7 @@ eio__sendfile (int ofd, int ifd, off_t offset, size_t count)
       if (sbytes)
         res = sbytes;
 
-# elif defined (__APPLE__)
+# elif defined __APPLE__
       off_t sbytes = count;
       res = sendfile (ifd, ofd, offset, &sbytes, 0, 0);
 
@@ -1384,6 +1385,20 @@ eio__mtouch (eio_req *req)
 /*****************************************************************************/
 /* requests implemented outside eio_execute, because they are so large */
 
+static void
+eio__lseek (eio_req *req)
+{
+  /* this usually gets optimised away completely, or your compiler sucks, */
+  /* or the whence constants really are not 0, 1, 2 */
+  int whence = req->int2 == EIO_SEEK_SET ? SEEK_SET
+             : req->int2 == EIO_SEEK_CUR ? SEEK_CUR
+             : req->int2 == EIO_SEEK_END ? SEEK_END
+             : req->int2;
+
+  req->offs   = lseek (req->int1, req->offs, whence);
+  req->result = req->offs == (off_t)-1 ? -1 : 0;
+}
+
 /* result will always end up in tmpbuf, there is always space for adding a 0-byte */
 static int
 eio__realpath (struct tmpbuf *tmpbuf, eio_wd wd, const char *path)
@@ -1489,7 +1504,7 @@ eio__realpath (struct tmpbuf *tmpbuf, eio_wd wd, const char *path)
 
         errno = ENAMETOOLONG;
         if (res + 1 + len + 1 >= tmp1)
-          return;
+          return -1;
 
         /* copy one component */
         *res = '/';
@@ -1968,7 +1983,7 @@ eio__scandir (eio_req *req, etp_worker *self)
                     {
                       if (*name == '.') /* leading dots are likely directories, and, in any case, rare */
                         ent->score = 1;
-                      else if (!strchr (name, '.')) /* absense of dots indicate likely dirs */
+                      else if (!strchr (name, '.')) /* absence of dots indicate likely dirs */
                         ent->score = len <= 2 ? 4 - len : len <= 4 ? 4 : len <= 7 ? 5 : 6; /* shorter == more likely dir, but avoid too many classes */
                     }
                   else if (ent->type == EIO_DT_DIR)
@@ -2316,6 +2331,7 @@ eio_execute (etp_worker *self, eio_req *req)
       case EIO_WD_CLOSE:  req->result = 0;
                           eio_wd_close_sync (req->wd); break;
 
+      case EIO_SEEK:      eio__lseek (req); break;
       case EIO_READ:      ALLOC (req->size);
                           req->result = req->offs >= 0
                                       ? pread     (req->int1, req->ptr2, req->size, req->offs)
@@ -2571,6 +2587,11 @@ eio_req *eio_readahead (int fd, off_t offset, size_t length, int pri, eio_cb cb,
   REQ (EIO_READAHEAD); req->int1 = fd; req->offs = offset; req->size = length; SEND;
 }
 
+eio_req *eio_seek (int fd, off_t offset, int whence, int pri, eio_cb cb, void *data)
+{
+  REQ (EIO_SEEK); req->int1 = fd; req->offs = offset; req->int2 = whence; SEND;
+}
+
 eio_req *eio_read (int fd, void *buf, size_t length, off_t offset, int pri, eio_cb cb, void *data)
 {
   REQ (EIO_READ); req->int1 = fd; req->offs = offset; req->size = length; req->ptr2 = buf; SEND;
